@@ -1,6 +1,4 @@
-/*
-   Copyright (c) 2000-2007 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (C) 2000 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
  * Vio Lite.
@@ -61,9 +58,6 @@ int vio_close_pipe(Vio * vio);
 #define HANDLE void *
 #endif /* __WIN__ */
 
-/* backport from 5.6 where it is part of PSI, not vio_*() */
-int	mysql_socket_shutdown(my_socket mysql_socket, int how);
-
 void	vio_delete(Vio* vio);
 int	vio_close(Vio* vio);
 void    vio_reset(Vio* vio, enum enum_vio_type type,
