@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,8 +22,8 @@ void my_sleep(ulong m_seconds)
 {
 #ifdef __NETWARE__
   delay(m_seconds/1000+1);
-#elif defined(OS2)
-  DosSleep(m_seconds/1000+1);
+#elif defined(__WIN__)
+  Sleep(m_seconds/1000+1);      /* Sleep() has millisecond arg */
 #elif defined(HAVE_SELECT)
   struct timeval t;
   t.tv_sec=  m_seconds / 1000000L;
